@@ -5,23 +5,7 @@ distribute_setup.use_setuptools()
 import setuptools
 from setuptools import Extension
 
-if 'distribute' not in setuptools.__file__:
-    print "-------------------------------------------------------------------------"
-    print "Setuptools conflict detected."
-    print "-------------------------------------------------------------------------"
-    print "When I imported setuptools, I did not get the distribute version of"
-    print "setuptools, which is troubling--this package really wants to be used"
-    print "with distribute rather than the old setuptools package. More than likely,"
-    print "you have both distribute and setuptools installed, which is bad."
-    print
-    print "See this page for more information:"
-    print "http://wiki.tiker.net/DistributeVsSetuptools"
-    print "-------------------------------------------------------------------------"
-    print "I will continue after a short while, fingers crossed."
-    print "-------------------------------------------------------------------------"
-
-    delay = 10
-
+def count_down_delay(delay):
     from time import sleep
     import sys
     while delay:
@@ -29,6 +13,29 @@ if 'distribute' not in setuptools.__file__:
         sys.stdout.flush()
         delay -= 1
         sleep(1)
+    print("")
+
+
+
+
+if not hasattr(setuptools, "_distribute"):
+    print("-------------------------------------------------------------------------")
+    print("Setuptools conflict detected.")
+    print("-------------------------------------------------------------------------")
+    print("When I imported setuptools, I did not get the distribute version of")
+    print("setuptools, which is troubling--this package really wants to be used")
+    print("with distribute rather than the old setuptools package. More than likely,")
+    print("you have both distribute and setuptools installed, which is bad.")
+    print("")
+    print("See this page for more information:")
+    print("http://wiki.tiker.net/DistributeVsSetuptools")
+    print("-------------------------------------------------------------------------")
+    print("I will continue after a short while, fingers crossed.")
+    print("Hit Ctrl-C now if you'd like to think about the situation.")
+    print("-------------------------------------------------------------------------")
+
+    count_down_delay(delay=10)
+
 
 def setup(*args, **kwargs):
     from setuptools import setup
@@ -40,16 +47,16 @@ def setup(*args, **kwargs):
     except SystemExit:
         raise
     except:
-        print "----------------------------------------------------------------------------"
-        print "Sorry, your build failed. Try rerunning configure.py with different options."
-        print "----------------------------------------------------------------------------"
+        print ("----------------------------------------------------------------------------")
+        print ("Sorry, your build failed. Try rerunning configure.py with different options.")
+        print ("----------------------------------------------------------------------------")
         raise
 
 
 
 
 class NumpyExtension(Extension):
-    # nicked from 
+    # nicked from
     # http://mail.python.org/pipermail/distutils-sig/2007-September/008253.html
     # solution by Michael Hoffmann
     def __init__(self, *args, **kwargs):
@@ -64,9 +71,14 @@ class NumpyExtension(Extension):
         from os.path import join
         return join(pathname, "core", "include")
 
-    @property
-    def include_dirs(self):
+    def get_include_dirs(self):
         return self._include_dirs + [self.get_numpy_incpath()]
+    def set_include_dirs(self, value):
+        self._include_dirs = value
+    def del_include_dirs(self):
+        pass
+
+    include_dirs = property(get_include_dirs, set_include_dirs, del_include_dirs)
 
 
 
@@ -109,7 +121,7 @@ class HedgeExtension(PyUblasExtension):
 
 # tools -----------------------------------------------------------------------
 def flatten(list):
-    """For an iterable of sub-iterables, generate each member of each 
+    """For an iterable of sub-iterables, generate each member of each
     sub-iterable in turn, i.e. a flattened version of that super-iterable.
 
     Example: Turn [[a,b,c],[d,e,f]] into [a,b,c,d,e,f].
@@ -136,37 +148,29 @@ def get_config(schema=None, warn_about_no_config=True):
 
     if (not schema.have_config() and not schema.have_global_config()
             and warn_about_no_config):
-        print "*************************************************************"
-        print "*** I have detected that you have not run configure.py."
-        print "*************************************************************"
-        print "*** Additionally, no global config files were found."
-        print "*** I will go ahead with the default configuration."
-        print "*** In all likelihood, this will not work out."
-        print "*** "
-        print "*** See README_SETUP.txt for more information."
-        print "*** "
-        print "*** If the build does fail, just re-run configure.py with the"
-        print "*** correct arguments, and then retry. Good luck!"
-        print "*************************************************************"
-        print "*** HIT Ctrl-C NOW IF THIS IS NOT WHAT YOU WANT"
-        print "*************************************************************"
+        print("*************************************************************")
+        print("*** I have detected that you have not run configure.py.")
+        print("*************************************************************")
+        print("*** Additionally, no global config files were found.")
+        print("*** I will go ahead with the default configuration.")
+        print("*** In all likelihood, this will not work out.")
+        print("*** ")
+        print("*** See README_SETUP.txt for more information.")
+        print("*** ")
+        print("*** If the build does fail, just re-run configure.py with the")
+        print("*** correct arguments, and then retry. Good luck!")
+        print("*************************************************************")
+        print("*** HIT Ctrl-C NOW IF THIS IS NOT WHAT YOU WANT")
+        print("*************************************************************")
 
-        delay = 10
+        count_down_delay(delay=10)
 
-        from time import sleep
-        import sys
-        while delay:
-            sys.stdout.write("Continuing in %d seconds...   \r" % delay)
-            sys.stdout.flush()
-            delay -= 1
-            sleep(1)
-
-    return schema.read_config()
+    return expand_options(schema.read_config())
 
 
 
 
-def hack_distutils(debug=False, fast_link=True):
+def hack_distutils(debug=False, fast_link=True, what_opt=3):
     # hack distutils.sysconfig to eliminate debug flags
     # stolen from mpi4py
 
@@ -190,8 +194,12 @@ def hack_distutils(debug=False, fast_link=True):
             if debug:
                 cflags.append("-g")
             else:
-                cflags.append("-O3")
-                cflags.append("-DNDEBUG")
+                if what_opt is None:
+                    pass
+                else:
+                    cflags.append("-O%s" % what_opt)
+                    cflags.append("-DNDEBUG")
+
             cvars['OPT'] = str.join(' ', cflags)
             cvars["CFLAGS"] = cvars["BASECFLAGS"] + " " + cvars["OPT"]
 
@@ -230,18 +238,26 @@ def expand_str(s, options):
     return re.subn(r"\$\{([a-zA-Z0-9_]+)\}", my_repl, s)[0]
 
 def expand_value(v, options):
-    if isinstance(v, (str, unicode)):
+    if isinstance(v, str):
         return expand_str(v, options)
     elif isinstance(v, list):
-        return [expand_value(i, options) for i in v]
+        result = []
+        for i in v:
+            try:
+                exp_i = expand_value(i, options)
+            except:
+                pass
+            else:
+                result.append(exp_i)
+
+        return result
     else:
         return v
 
 
 def expand_options(options):
-    for k in options.keys():
-        options[k] = expand_value(options[k], options)
-    return options
+    return dict(
+            (k, expand_value(v, options)) for k, v in options.items())
 
 
 
@@ -272,15 +288,20 @@ class ConfigSchema:
         self.conf_dir = conf_dir
 
     def get_default_config(self):
-        return dict((opt.name, opt.default) 
-                for opt in self.options)
-        
+        return dict((opt.name, opt.default) for opt in self.options)
+
     def read_config_from_pyfile(self, filename):
         result = {}
         filevars = {}
-        execfile(filename, filevars)
+        infile = open(filename, "r")
+        try:
+            contents = infile.read()
+        finally:
+            infile.close()
 
-        for key, value in filevars.iteritems():
+        exec(compile(contents, filename, "exec"), filevars)
+
+        for key, value in filevars.items():
             if key in self.optdict:
                 result[key] = value
 
@@ -291,13 +312,14 @@ class ConfigSchema:
         filevars = {}
 
         try:
-            execfile(filename, filevars)
+            exec(compile(open(filename, "r").read(), filename, "exec"), filevars)
         except IOError:
             pass
 
-        del filevars["__builtins__"]
+        if "__builtins__" in filevars:
+            del filevars["__builtins__"]
 
-        for key, value in config.iteritems():
+        for key, value in config.items():
             if value is not None:
                 filevars[key] = value
 
@@ -322,7 +344,7 @@ class ConfigSchema:
         result = self.get_default_config()
 
         import os
-        
+
         confignames = []
         if self.global_conf_file is not None:
             confignames.append(self.global_conf_file)
@@ -354,18 +376,16 @@ class ConfigSchema:
         result = self.get_default_config_with_files()
         if os.access(cfile, os.R_OK):
             filevars = {}
-            execfile(cfile, filevars)
+            exec(compile(open(cfile, "r").read(), cfile, "exec"), filevars)
 
-            for key, value in filevars.iteritems():
+            for key, value in filevars.items():
                 if key in self.optdict:
                     result[key] = value
                 elif key == "__builtins__":
                     pass
                 else:
-                    raise KeyError, "invalid config key in %s: %s" % (
-                            cfile, key)
-
-        expand_options(result)
+                    raise KeyError("invalid config key in %s: %s" % (
+                            cfile, key))
 
         return result
 
@@ -381,11 +401,9 @@ class ConfigSchema:
         result = {}
         for opt in self.options:
             result[opt.name] = opt.take_from_configparser(options)
-        expand_options(result)
         return result
 
     def write_config(self, config):
-        import os
         outf = open(self.get_conf_file(), "w")
         for opt in self.options:
             value = config[opt.name]
@@ -425,13 +443,13 @@ class Option(object):
         return result
 
     def value_to_str(self, default):
-        return default 
+        return default
 
     def add_to_configparser(self, parser, default=None):
         default = default_or(default, self.default)
         default_str = self.value_to_str(default)
         parser.add_option(
-            "--" + self.as_option(), dest=self.name, 
+            "--" + self.as_option(), dest=self.name,
             default=default_str,
             metavar=self.metavar(), help=self.get_help(default))
 
@@ -443,20 +461,23 @@ class Switch(Option):
         option = self.as_option()
 
         if not isinstance(self.default, bool):
-            raise ValueError, "Switch options must have a default"
+            raise ValueError("Switch options must have a default")
 
         if default is None:
             default = self.default
 
+        option_name = self.as_option()
+
         if default:
+            option_name = "no-" + option_name
             action = "store_false"
         else:
             action = "store_true"
-            
+
         parser.add_option(
-            "--" + self.as_option(), 
-            dest=self.name, 
-            help=self.get_help(default), 
+            "--" + option_name,
+            dest=self.name,
+            help=self.get_help(default),
             default=default,
             action=action)
 
@@ -465,7 +486,7 @@ class StringListOption(Option):
         if default is None:
             return None
 
-        return ",".join([str(el) for el in default])
+        return ",".join([str(el).replace(",", r"\,") for el in default])
 
     def get_help(self, default):
         return Option.get_help(self, default) + " (several ok)"
@@ -476,7 +497,11 @@ class StringListOption(Option):
             return None
         else:
             if opt:
-                return opt.split(",")
+                import re
+                sep = re.compile(r"(?<!\\),")
+                result = sep.split(opt)
+                result = [i.replace(r"\,", ",") for i in result]
+                return result
             else:
                 return []
 
@@ -484,7 +509,7 @@ class StringListOption(Option):
 class IncludeDir(StringListOption):
     def __init__(self, lib_name, default=None, human_name=None, help=None):
         StringListOption.__init__(self, "%s_INC_DIR" % lib_name, default,
-                help=help or ("Include directories for %s" 
+                help=help or ("Include directories for %s"
                 % (human_name or humanize(lib_name))))
 
 class LibraryDir(StringListOption):
@@ -496,17 +521,17 @@ class LibraryDir(StringListOption):
 class Libraries(StringListOption):
     def __init__(self, lib_name, default=None, human_name=None, help=None):
         StringListOption.__init__(self, "%s_LIBNAME" % lib_name, default,
-                help=help or ("Library names for %s (without lib or .so)" 
+                help=help or ("Library names for %s (without lib or .so)"
                 % (human_name or humanize(lib_name))))
 
 class BoostLibraries(Libraries):
     def __init__(self, lib_base_name):
-        Libraries.__init__(self, "BOOST_%s" % lib_base_name.upper(), 
-                ["boost_%s-${BOOST_COMPILER}-mt" % lib_base_name],
-                help="Library names for Boost C++ %s library (without lib or .so)" 
+        Libraries.__init__(self, "BOOST_%s" % lib_base_name.upper(),
+                ["boost_%s" % lib_base_name],
+                help="Library names for Boost C++ %s library (without lib or .so)"
                     % humanize(lib_base_name))
 
-def set_up_shipped_boost_if_requested(conf):
+def set_up_shipped_boost_if_requested(project_name, conf):
     """Set up the package to use a shipped version of Boost.
 
     Return a tuple of a list of extra C files to build and extra
@@ -517,33 +542,25 @@ def set_up_shipped_boost_if_requested(conf):
 
     if conf["USE_SHIPPED_BOOST"]:
         if not exists("bpl-subset/bpl_subset/boost/version.hpp"):
-            print >>sys.stderr, "------------------------------------------------------------------------"
-            print >>sys.stderr, "The shipped Boost library was not found, but USE_SHIPPED_BOOST is True."
-            print >>sys.stderr, "(The files should be under bpl-subset/.)"
-            print >>sys.stderr, "------------------------------------------------------------------------"
-            print >>sys.stderr, "If you got this package from git, you probably want to do"
-            print >>sys.stderr, ""
-            print >>sys.stderr, " $ git submodule init"
-            print >>sys.stderr, " $ git submodule update"
-            print >>sys.stderr, ""
-            print >>sys.stderr, "to fetch what you are presently missing. If you got this from"
-            print >>sys.stderr, "a distributed package on the net, that package is broken and"
-            print >>sys.stderr, "should be fixed. For now, I will turn off 'USE_SHIPPED_BOOST'"
-            print >>sys.stderr, "to try and see if the build succeeds that way, but in the long"
-            print >>sys.stderr, "run you might want to either get the missing bits or turn"
-            print >>sys.stderr, "'USE_SHIPPED_BOOST' off."
-            print >>sys.stderr, "------------------------------------------------------------------------"
+            print("------------------------------------------------------------------------")
+            print("The shipped Boost library was not found, but USE_SHIPPED_BOOST is True.")
+            print("(The files should be under bpl-subset/.)")
+            print("------------------------------------------------------------------------")
+            print("If you got this package from git, you probably want to do")
+            print("")
+            print(" $ git submodule init")
+            print(" $ git submodule update")
+            print("")
+            print("to fetch what you are presently missing. If you got this from")
+            print("a distributed package on the net, that package is broken and")
+            print("should be fixed. For now, I will turn off 'USE_SHIPPED_BOOST'")
+            print("to try and see if the build succeeds that way, but in the long")
+            print("run you might want to either get the missing bits or turn")
+            print("'USE_SHIPPED_BOOST' off.")
+            print("------------------------------------------------------------------------")
             conf["USE_SHIPPED_BOOST"] = False
 
-            delay = 10
-
-            from time import sleep
-            import sys
-            while delay:
-                sys.stdout.write("Continuing in %d seconds...   \r" % delay)
-                sys.stdout.flush()
-                delay -= 1
-                sleep(1)
+            count_down_delay(delay=10)
 
     if conf["USE_SHIPPED_BOOST"]:
         conf["BOOST_INC_DIR"] = ["bpl-subset/bpl_subset"]
@@ -556,19 +573,46 @@ def set_up_shipped_boost_if_requested(conf):
                 + glob("bpl-subset/bpl_subset/libs/*/*/*.cpp")
                 + glob("bpl-subset/bpl_subset/libs/*/*.cpp"))
 
+        # make sure next line succeeds even on Windows
+        source_files = [f.replace("\\","/") for f in source_files]
+
         source_files = [f for f in source_files
                 if not f.startswith("bpl-subset/bpl_subset/libs/thread/src")]
 
-        import sys
-        if sys.platform == "nt":
+        if sys.platform == "win32":
             source_files += glob(
                     "bpl-subset/bpl_subset/libs/thread/src/win32/*.cpp")
+            source_files += glob(
+                    "bpl-subset/bpl_subset/libs/thread/src/*.cpp")
         else:
             source_files += glob(
                     "bpl-subset/bpl_subset/libs/thread/src/pthread/*.cpp")
 
-        return (source_files, 
-                {"BOOST_MULTI_INDEX_DISABLE_SERIALIZATION": 1}
+        from os.path import isdir
+        main_boost_inc = "bpl-subset/bpl_subset/boost"
+        bpl_project_boost_inc = "bpl-subset/bpl_subset/%sboost" % project_name
+
+        if not isdir(bpl_project_boost_inc):
+            try:
+                from os import symlink
+            except ImportError:
+                from shutil import copytree
+                print("Copying files, hang on... (do not interrupt)")
+                copytree(main_boost_inc, bpl_project_boost_inc)
+            else:
+                symlink("boost", bpl_project_boost_inc)
+
+        return (source_files,
+                {
+                    # do not pick up libboost link dependency on windows
+                    "BOOST_ALL_NO_LIB": 1,
+                    "BOOST_THREAD_BUILD_DLL": 1,
+
+                    "BOOST_MULTI_INDEX_DISABLE_SERIALIZATION": 1,
+                    "BOOST_THREAD_DONT_USE_CHRONO": 1,
+                    "BOOST_PYTHON_SOURCE": 1,
+                    "boost": '%sboost' % project_name
+                    }
                 )
     else:
         return [], {}
@@ -578,7 +622,7 @@ def make_boost_base_options():
     return [
         IncludeDir("BOOST", []),
         LibraryDir("BOOST", []),
-        Option("BOOST_COMPILER", default="gcc43", 
+        Option("BOOST_COMPILER", default="gcc43",
             help="The compiler with which Boost C++ was compiled, e.g. gcc43"),
         ]
 
@@ -594,29 +638,29 @@ def configure_frontend():
     from setup import get_config_schema
     schema = get_config_schema()
     if schema.have_config():
-        print "************************************************************"
-        print "*** I have detected that you have already run configure."
-        print "*** I'm taking the configured values as defaults for this"
-        print "*** configure run. If you don't want this, delete the file"
-        print "*** %s." % schema.get_conf_file()
-        print "************************************************************"
+        print("************************************************************")
+        print("*** I have detected that you have already run configure.")
+        print("*** I'm taking the configured values as defaults for this")
+        print("*** configure run. If you don't want this, delete the file")
+        print("*** %s." % schema.get_conf_file())
+        print("************************************************************")
 
     import sys
 
     description = "generate a configuration file for this software package"
     parser = OptionParser(description=description)
     parser.add_option(
-	    "--python-exe", dest="python_exe", default=sys.executable,
-	    help="Which Python interpreter to use", metavar="PATH")
+            "--python-exe", dest="python_exe", default=sys.executable,
+            help="Which Python interpreter to use", metavar="PATH")
 
     parser.add_option("--prefix", default=None,
-	    help="Ignored")
+            help="Ignored")
     parser.add_option("--enable-shared", help="Ignored", action="store_false")
     parser.add_option("--disable-static", help="Ignored", action="store_false")
-    parser.add_option("--update-user", help="Update user config file (%s)" % schema.user_conf_file, 
+    parser.add_option("--update-user", help="Update user config file (%s)" % schema.user_conf_file,
             action="store_true")
-    parser.add_option("--update-global", 
-            help="Update global config file (%s)" % schema.global_conf_file, 
+    parser.add_option("--update-global",
+            help="Update global config file (%s)" % schema.global_conf_file,
             action="store_true")
 
     schema.add_to_configparser(parser, schema.read_config())
@@ -680,3 +724,103 @@ def substitute(substitutions, fname):
     from os import stat, chmod
     infile_stat_res = stat(fname_in)
     chmod(fname, infile_stat_res.st_mode)
+
+
+
+
+def check_git_submodules():
+    from os.path import isdir
+    if not isdir(".git"):
+        # not a git repository
+        return
+
+    git_error = None
+    from subprocess import Popen, PIPE
+    try:
+        popen = Popen(["git", "--version"], stdout=PIPE)
+        stdout_data, _ = popen.communicate()
+        if popen.returncode != 0:
+            git_error = "git returned error code %d" % popen.returncode
+    except OSError:
+        git_error = "(os error, likely git not found)"
+
+    if git_error is not None:
+        print("-------------------------------------------------------------------------")
+        print("Trouble invoking git")
+        print("-------------------------------------------------------------------------")
+        print("The package directory appears to be a git repository, but I could")
+        print("not invoke git to check whether my submodules are up to date.")
+        print("")
+        print("The error was:")
+        print(git_error)
+        print("Hit Ctrl-C now if you'd like to think about the situation.")
+        print("-------------------------------------------------------------------------")
+        count_down_delay(delay=5)
+        return
+
+    popen = Popen(["git", "submodule", "status"], stdout=PIPE)
+    stdout_data, _ = popen.communicate()
+    stdout_data = stdout_data.decode("ascii")
+    if popen.returncode != 0:
+        git_error = "git returned error code %d" % popen.returncode
+
+    pkg_warnings = []
+
+    lines = stdout_data.split("\n")
+    for l in lines:
+        if not l.strip():
+            continue
+
+        status = l[0]
+        sha, package = l[1:].split(" ", 1)
+
+        if package == "bpl-subset":
+            # treated separately
+            continue
+
+        if status == "+":
+            pkg_warnings.append("version of '%s' is not what this outer package wants"
+                    % package)
+        elif status == "-":
+            pkg_warnings.append("subpackage '%s' is not initialized" 
+                    % package)
+        elif status == " ":
+            pass
+        else:
+            pkg_warnings.append("subpackage '%s' has unrecognized status '%s'"
+                    % package)
+
+    if pkg_warnings:
+            print("-------------------------------------------------------------------------")
+            print("git submodules are not up-to-date or in odd state")
+            print("-------------------------------------------------------------------------")
+            print("If this makes no sense, you probably want to say")
+            print("")
+            print(" $ git submodule init")
+            print(" $ git submodule update")
+            print("")
+            print("to fetch what you are presently missing and move on with your life.")
+            print("If you got this from a distributed package on the net, that package is")
+            print("broken and should be fixed. Please inform whoever gave you this package.")
+            print("")
+            print("These issues were found:")
+            for w in pkg_warnings:
+                print("  %s" % w)
+            print("")
+            print("I will try to continue after a short wait, fingers crossed.")
+            print("-------------------------------------------------------------------------")
+            print("Hit Ctrl-C now if you'd like to think about the situation.")
+            print("-------------------------------------------------------------------------")
+
+            from os.path import exists
+            if not exists(".dirty-git-ok"):
+                count_down_delay(delay=10)
+
+
+
+
+
+
+
+
+
